@@ -54,7 +54,8 @@ ImageView::ImageView(QWidget* parent):
   isSVG_(false),
   currentTool_(ToolNone),
   nextNumber_(1),
-  showOutline_(false) {
+  showOutline_(false),
+  tiledBg_(false) {
 
   setViewportMargins(0, 0, 0, 0);
   setContentsMargins(0, 0, 0, 0);
@@ -360,12 +361,16 @@ void ImageView::flipImage(bool horizontal) {
     }
   }
   if(!image_.isNull()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6,9,0))
+    image_ = image_.flipped(horizontal ? Qt::Horizontal : Qt::Vertical);
+#else
     if(horizontal) {
       image_ = image_.mirrored(true, false);
     }
     else {
       image_ = image_.mirrored(false, true);
     }
+#endif
     int tmp = nextNumber_;
     setImage(image_, !gifMovie_ && !isSVG_, false);
     nextNumber_ = tmp;
@@ -437,7 +442,7 @@ bool ImageView::resizeImage(const QSize& newSize) {
 
 void ImageView::drawOutline() {
   QColor col = QColor(Qt::black);
-  if(qGray(backgroundBrush().color().rgb()) < GRAY) {
+  if(!tiledBg_ && qGray(backgroundBrush().color().rgb()) < GRAY) {
     col = QColor(Qt::white);
   }
   QPen outline(col, 1, Qt::DashLine);
@@ -448,6 +453,36 @@ void ImageView::drawOutline() {
   outlineItem_->setZValue(1); // to be drawn on top of all other items
 }
 
+void ImageView::setViewBackground(const QBrush& brush, bool solidBg) {
+  tiledBg_ = !solidBg;
+  QColor col;
+  if(solidBg) {
+    setBackgroundBrush(brush);
+    if(outlineItem_) {
+      col = qGray(brush.color().rgb()) < GRAY ? QColor(Qt::white) : QColor(Qt::black);
+    }
+  }
+  else {
+    QPixmap tilePixmap(64, 64);
+    tilePixmap.fill(Qt::white);
+    QPainter tilePainter(&tilePixmap);
+    QColor color(220, 220, 220);
+    tilePainter.fillRect(0, 0, 32, 32, color);
+    tilePainter.fillRect(32, 32, 32, 32, color);
+    tilePainter.end();
+    setBackgroundBrush(tilePixmap);
+    if(outlineItem_) {
+      col = QColor(Qt::black);
+    }
+  }
+  if(col.isValid()) {
+    QPen outline = outlineItem_->pen();
+    outline.setColor(col);
+    outlineItem_->setPen(outline);
+    viewport()->update();
+  }
+}
+
 void ImageView::setImage(const QImage& image, bool show, bool updatePixelRatio) {
   if(show) {
     resetView();
@@ -502,7 +537,72 @@ void ImageView::setImage(const QImage& image, bool show, bool updatePixelRatio)
   queueGenerateCache();
 }
 
-void ImageView::setGifAnimation(const QString& fileName) {
+bool ImageView::supportsAnimation(const QString& fileName) const {
+  QMovie movie(fileName);
+  return movie.frameCount() > 1;
+}
+
+int ImageView::frameCount() const {
+  if(gifMovie_) {
+    return gifMovie_->frameCount();
+  }
+  return 0;
+}
+
+int ImageView::currentFrame() const {
+  if(gifMovie_) {
+    return gifMovie_->currentFrameNumber() + 1;
+  }
+  return 0;
+}
+
+void ImageView::nextFrame() {
+  if(gifMovie_ && gifMovie_->frameCount() > 1) {
+    removeAnnotations();
+    int curFrame = gifMovie_->currentFrameNumber();
+    gifMovie_->jumpToFrame(curFrame < gifMovie_->frameCount() - 1 ? curFrame + 1 : 0);
+    image_ = gifMovie_->currentImage();
+    if(QGraphicsItem* imageItem = imageGraphicsItem()) {
+        image_ = image_.transformed(imageItem->transform(), Qt::SmoothTransformation);
+    }
+  }
+}
+
+void ImageView::previousFrame() {
+  if(gifMovie_ && gifMovie_->frameCount() > 1) {
+    removeAnnotations();
+    int curFrame = gifMovie_->currentFrameNumber();
+    gifMovie_->jumpToFrame(curFrame > 0 ? curFrame - 1 : gifMovie_->frameCount() - 1);
+    image_ = gifMovie_->currentImage();
+    if(QGraphicsItem* imageItem = imageGraphicsItem()) {
+        image_ = image_.transformed(imageItem->transform(), Qt::SmoothTransformation);
+    }
+  }
+}
+
+void ImageView::firstFrame() {
+  if(gifMovie_ && gifMovie_->currentFrameNumber() > 0) {
+    removeAnnotations();
+    gifMovie_->jumpToFrame(0);
+    image_ = gifMovie_->currentImage();
+    if(QGraphicsItem* imageItem = imageGraphicsItem()) {
+        image_ = image_.transformed(imageItem->transform(), Qt::SmoothTransformation);
+    }
+  }
+}
+
+void ImageView::lastFrame() {
+  if(gifMovie_ && gifMovie_->currentFrameNumber() < gifMovie_->frameCount() - 1) {
+    removeAnnotations();
+    gifMovie_->jumpToFrame(gifMovie_->frameCount() - 1);
+    image_ = gifMovie_->currentImage();
+    if(QGraphicsItem* imageItem = imageGraphicsItem()) {
+        image_ = image_.transformed(imageItem->transform(), Qt::SmoothTransformation);
+    }
+  }
+}
+
+void ImageView::setGifAnimation(const QString& fileName, bool startAnimation) {
   resetView();
   /* the built-in gif reader gives the first frame, which won't
      be shown but is used for tracking position and dimensions */
@@ -540,7 +640,12 @@ void ImageView::setGifAnimation(const QString& fileName) {
     gifLabel->setAttribute(Qt::WA_NoSystemBackground);
     gifLabel->setMovie(gifMovie_);
     gifWidget->setWidget(gifLabel);
-    gifMovie_->start();
+    if(startAnimation) {
+      gifMovie_->start();
+    }
+    else {
+      gifMovie_->jumpToFrame(0);
+    }
     scene_->addItem(gifItem);
     scene_->setSceneRect(gifItem->boundingRect());
 
@@ -613,19 +718,6 @@ void ImageView::showOutline(bool show) {
   showOutline_ = show;
 }
 
-void ImageView::updateOutline() {
-  if(outlineItem_) {
-    QColor col = QColor(Qt::black);
-    if(qGray(backgroundBrush().color().rgb()) < GRAY) {
-      col = QColor(Qt::white);
-    }
-    QPen outline = outlineItem_->pen();
-    outline.setColor(col);
-    outlineItem_->setPen(outline);
-    viewport()->update();
-  }
-}
-
 void ImageView::paintEvent(QPaintEvent* event) {
   if (!smoothOnZoom_) {
     QGraphicsView::paintEvent(event);
@@ -649,7 +741,7 @@ void ImageView::paintEvent(QPaintEvent* event) {
         // outline
         if(showOutline_) {
             QColor col = QColor(Qt::black);
-            if(qGray(backgroundBrush().color().rgb()) < GRAY) {
+            if(!tiledBg_ && qGray(backgroundBrush().color().rgb()) < GRAY) {
               col = QColor(Qt::white);
             }
             QPen outline(col, 1, Qt::DashLine);
@@ -666,6 +758,17 @@ void ImageView::paintEvent(QPaintEvent* event) {
   QGraphicsView::paintEvent(event);
 }
 
+void ImageView::drawBackground(QPainter *p, const QRectF& rect) {
+  if(!tiledBg_) {
+    QGraphicsView::drawBackground(p, rect);
+    return;
+  }
+  p->save();
+  p->resetTransform();
+  p->drawTiledPixmap(viewport()->rect(), backgroundBrush().texture());
+  p->restore();
+}
+
 void ImageView::queueGenerateCache() {
   if(!cachedPixmap_.isNull()) // clear the old pixmap if there's any
     cachedPixmap_ = QPixmap();
@@ -822,6 +925,10 @@ void ImageView::resetView() {
     }
   }
   // remove annotations
+  removeAnnotations();
+}
+
+void ImageView::removeAnnotations() {
   if(!annotations_.isEmpty()) {
     if(!scene_->items().isEmpty()) { // WARNING: This is not enough to guard against dangling pointers.
       for(const auto& annotation : std::as_const(annotations_)) {
